﻿//---------------------------------------------------------------------------
//
//   VertexMarker
//
//          Copyright(C) 2014, Tark
//
//  頂点・辺にカラーマーク
//
//
//---------------------------------------------------------------------------

#include <windows.h>
//#include <stdio.h>
//#include <math.h>
//#include <unordered_map>
//#include <unordered_set>
#include <algorithm>
#include "MQPlugin.h"

//#include "MQ3DLib.h"
#include "MQWidget.h"
#include "MQSetting.h"

#include "CMQObject.h"
#include "VertexMarkerWindow.h"
#include "VertexMarker.h"

static VertexMarker plugin;

//---------------------------------------------------------------------------
//  MQGetPlugInID
//---------------------------------------------------------------------------
void VertexMarker::GetPlugInID(DWORD *Product, DWORD *ID)
{
	*Product = 0xE3F7A830;
	*ID      = 0xBD9A1883;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//---------------------------------------------------------------------------
const char *VertexMarker::GetPlugInName(void)
{
	return "VertexMarker ver.1.0 Copyright(C) 2014, Tark.";
}

//---------------------------------------------------------------------------
//  MQEnumString
//---------------------------------------------------------------------------
const char *VertexMarker::EnumString()
{
	return "VertexMarker";
}

//---------------------------------------------------------------------------
//  Initialize
//---------------------------------------------------------------------------
BOOL VertexMarker::Initialize()
{
	int posX = -1, posY = -1;
	m_marked.resize(3);
	{
		MQSetting *setting = OpenSetting();
		setting->Load("VertexMarker_activate", m_bActivate, false);
		setting->Load("PosX", posX, -1);
		setting->Load("PosY", posY, -1);
		char key[3];
		for(int i = 0; i < 3; i++) {
			sprintf_s(key, "r%d", i);
			setting->Load(key, m_marked[i].m_color.r, 1.0f);
			sprintf_s(key, "g%d", i);
			setting->Load(key, m_marked[i].m_color.g, 0.0f);
			sprintf_s(key, "b%d", i);
			setting->Load(key, m_marked[i].m_color.b, 1.0f);
		}
		CloseSetting(setting);
	}
	if(m_Window == NULL) {
		MQWindowBase mainwnd = MQWindow::GetMainWindow();
		m_Window = new VertexMarkerWindow(this, mainwnd);
		if(posX >= 0 && posY >= 0) {
			m_Window->SetPosX(posX);
			m_Window->SetPosY(posY);
		}
	}
	Activate(nullptr, m_bActivate);
	return TRUE;
}

//---------------------------------------------------------------------------
//  Exit
//---------------------------------------------------------------------------
void VertexMarker::Exit()
{
	{
		MQSetting *setting = OpenSetting();
		setting->Save("VertexMarker_activate", m_bActivate);
		setting->Save("PosX", m_Window->GetPosX());
		setting->Save("PosY", m_Window->GetPosY());
		char key[3];
		for(int i = 0; i < 3; i++) {
			sprintf_s(key, "r%d", i);
			setting->Save(key, m_marked[i].m_color.r);
			sprintf_s(key, "g%d", i);
			setting->Save(key, m_marked[i].m_color.g);
			sprintf_s(key, "b%d", i);
			setting->Save(key, m_marked[i].m_color.b);
		}
		CloseSetting(setting);
	}
	if(m_Window != NULL) {
		delete m_Window;
		m_Window = NULL;
	}
}

//---------------------------------------------------------------------------
//  Activate
//---------------------------------------------------------------------------
BOOL VertexMarker::Activate(MQDocument doc, BOOL flag)
{
	if (!m_Window) {
		return FALSE;
	}
	m_bActivate = flag ? true : false;
	m_Window->SetVisible(m_bActivate);
	if(m_bActivate) {
		m_normal_update = true;
		MQ_RefreshView(nullptr);
	}
	else {
		//DeleteDrawingObject(m_doc, m_draw);
		//m_draw = nullptr;
		//m_obj = nullptr;
		m_doc = nullptr;
	}
	return m_bActivate;
}

//---------------------------------------------------------------------------
//  IsActivated
//---------------------------------------------------------------------------
BOOL VertexMarker::IsActivated(MQDocument doc)
{
	if (m_Window == NULL) {
		return FALSE;
	}
	return m_bActivate;
}

//---------------------------------------------------------------------------
//  ExecuteCallback
//---------------------------------------------------------------------------
bool VertexMarker::ExecuteCallback(MQDocument doc, void *option)
{
	CallbackInfo *info = (CallbackInfo*)option;
	return ((*this).*info->proc)(doc);
}

// コールバックの呼び出し
void VertexMarker::Execute(ExecuteCallbackProc proc)
{
	CallbackInfo info;
	info.proc = proc;
	BeginCallback(&info);
}

// プラグインのベースクラスを返す
MQBasePlugin *GetPluginClass()
{
	return &plugin;
}

VertexMarker::VertexMarker() :
		m_Window(nullptr), m_bActivate(false),
		m_doc(nullptr), //m_obj(nullptr), //m_draw(nullptr),
		m_view(true), m_normal_update(true),
		m_draw_normal(false), m_draw_update(true),
		m_normal_length(10.0f), m_overwrite_selected(true)
{
}

void VertexMarker::DeleteDrawingObject_(MQDocument doc, Marked &m)
{
	DeleteDrawingObject(doc, m.m_draw);
	m.m_draw = nullptr;
}

VertexMarker::~VertexMarker()
{
	for(auto &m : m_marked) {
		if(m.m_draw) {
			DeleteDrawingObject_(m_doc, m);
		}
	}
	if(!m_Window){
		delete m_Window;
		m_Window = nullptr;
	}
}

void VertexMarker::OnNewDocument(MQDocument doc, const char *filename, NEW_DOCUMENT_PARAM& param)
{
	m_Window->resetall();
	for(auto &m : m_marked) {
		if(m.m_draw) {
			DeleteDrawingObject_(m_doc, m);
		}
		m.m_normal.clear();
		m.m_lines.clear();
		m.m_vertices.clear();
	}
}

void VertexMarker::createNormalViewObject()
{
	cmq::CMQDocument doc(m_doc);
	for(auto &mark : m_marked) {
		UINT curobjuid = mark.m_objuid;
		cmq::CMQObject obj = doc.getObjectFromUniqueID(curobjuid);
		if(!obj.isLive() || !obj.getVisible()) {
			continue;
		}
		mark.m_normal.clear();
		// 面の法線
		cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
		for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
			UINT vuid = obj.getVertexUniqueID(vit.index());
			// 法線を計算する対象か（マークした頂点かどうか）
			// 頂点
			if(std::find(mark.m_vertices.begin(), mark.m_vertices.end(), vuid) == mark.m_vertices.end()) {
				// 辺
				if(std::find_if(mark.m_lines.begin(), mark.m_lines.end(), [&obj, &vuid](cmq::lindex_t lindex) {
					auto vpair = cmq::lineId2VertexPair(lindex);
					return vpair.first == vuid || vpair.second == vuid;
				}) == mark.m_lines.end()) {
					// セットされた頂点でも辺でもない
					continue;
				}
			}
			MQPoint normal(0, 0, 0);
			int normalcount = 0;
			std::vector<cmq::findex_t> farray(5);
			obj.getVertexRelatedFaces(vit.index(), farray);
			for(auto findex : farray) {
				// 法線計算
				auto varray = obj.getFacePointArray(findex);
				auto findit = std::find(varray.begin(), varray.end(), vit.index());
				if(findit == varray.end()) {
					//something wrong
					continue;
				}
				MQPoint nvec = obj.getNormalVector(findex, *findit);
				nvec.normalize();
				normal += nvec;
				normalcount++;
			}

			if(normalcount > 0) {
				normal /= (float)normalcount;
				mark.m_normal.insert(std::make_pair(vuid, normal));
			}
		}
	}
	m_normal_update = false;
}

void VertexMarker::OnDraw(MQDocument doc_, MQScene scene, int width, int height)
{
	if(!IsActivated(doc_)) {
		return;
	}
	m_doc = doc_;
	cmq::CMQDocument doc(m_doc);
	
	SCENE_OPTION option;
	GetSceneOption(scene, option);
	DRAW_OBJECT_VIISIBILITY draw_object_visibility;
	draw_object_visibility = static_cast<DRAW_OBJECT_VIISIBILITY>((option.ShowVertex ? DRAW_OBJECT_POINT : 0) | (m_view_always | option.ShowLine ? DRAW_OBJECT_LINE : 0));

	if(m_draw_normal && m_normal_update) {
		createNormalViewObject();
	}

	for(auto it = m_marked.rbegin(); it != m_marked.rend(); ++it) {
		// 各マークオブジェクトについて、後ろから順に
		Marked &m = *it;
		if(!m_view) {
			// 表示しない
			DeleteDrawingObject_(doc_, m);
			continue;
		}
		cmq::CMQObject obj = doc.getObjectFromUniqueID(m.m_objuid);
		if(!obj.isLive() || !obj.getVisible()) {
			// 非表示オブジェクト
			DeleteDrawingObject_(doc_, m);
			continue;
		}
		if(!m.m_update && !m_draw_update) {
			// 更新不必要
			continue;
		}
		// DrawingObject作成
		if(m.m_draw) {
			DeleteDrawingObject_(doc_, m);
		}
		if(m.empty() || !m.m_markon) {
			continue;
		}
		m.m_draw = CreateDrawingObject(m_doc, draw_object_visibility, FALSE);
		m.m_draw->SetColor(m.m_color);
		m.m_draw->SetColorValid(TRUE);
		cmq::CMQObject draw(m.m_draw, doc);

		// 着色で作成した頂点を格納する(対象オブジェクト頂点index→Drawオブジェクト頂点index)
		std::unordered_map<cmq::vindex_t, cmq::vindex_t> created_vertex;
		// 着色で作成しなかった頂点
		std::unordered_map<cmq::vindex_t, cmq::vindex_t> nocreated_vertex;

		// 辺着色
		for(auto lid : m.m_lines) {
			std::pair<UINT, UINT> lpair = cmq::lineId2VertexPair(lid);
			cmq::vindex_t v[2];
			v[0] = obj.getVertexIndexFromUniqueID(lpair.first);
			v[1] = obj.getVertexIndexFromUniqueID(lpair.second);
			if(v[0] < 0 || v[1] < 0) {
				continue;
			}
			bool selected_flag = false;
			if(!m_overwrite_selected) {
				// 選択面または選択辺なら描画しない
				auto lines = obj.getNeighborLine(v[0], v[1]);
				auto it = std::find_if(lines.begin(), lines.end(), [&doc, &obj](cmq::flindex_t flindex) -> bool {
					auto flpair = cmq::flId2FaceLinePair(flindex);
					return doc.isSelectFace(obj, flpair.first) || doc.isSelectLine(obj, flpair.first, flpair.second);
				});
				if(it != lines.end()) {
					selected_flag = true;
					//continue;
				}
			}
			cmq::vindex_t drawv[2];
			for(int i = 0; i < 2; i++) {
				auto findit = created_vertex.find(v[i]);
				if(findit != created_vertex.end()) {
					// 作成済み頂点
					drawv[i] = findit->second;
				}
				else {
					// 未作成頂点
					drawv[i] = draw.addVertex(obj.getVertex(v[i]));
					created_vertex.insert(std::make_pair(v[i], drawv[i]));
				}
			}
			if(!selected_flag) {
				draw.createLine(drawv[0], drawv[1]);
			}
		}

		// 頂点着色
		for(auto vuid : m.m_vertices) {
			cmq::vindex_t vidx = obj.getVertexIndexFromUniqueID(vuid);
			if(vidx < 0) {
				continue;
			}
			if(created_vertex.find(vidx) != created_vertex.end()) {
				// 既に作成済み
				continue;
			}
			bool selected_flag = false;
			if(!m_overwrite_selected && obj.isSelectVertex(vidx)) {
				// 選択頂点なら描画しない
				selected_flag = true;
				//continue;
			}
			MQPoint p1 = obj.getVertex(vidx);
			MQPoint p2(p1);
			cmq::vindex_t added_vertex[2];
			if(!selected_flag) {
				draw.createLine(p1, p2, added_vertex);
			}
			created_vertex.insert(std::make_pair(vidx, added_vertex[0]));
		}

		// 法線作成(選択辺/頂点で作成しなかったかに関わらず法線は作成する)
		if(m_draw_normal) {
			for(auto &vpair : created_vertex) {
				cmq::vindex_t v1 = vpair.first;
				UINT vuid = obj.getVertexUniqueID(v1);
				cmq::vindex_t v2 = vpair.second;
				auto findit = m.m_normal.find(vuid);
				if(findit != m.m_normal.end()) {
					MQPoint p1 = obj.getVertex(v1);
					MQPoint p2 = p1 + findit->second * m_normal_length;
					draw.createLine(v2, p2);
				}
			}
		}

		m.m_update = false;
	}
}

BOOL VertexMarker::OnUndo(MQDocument doc, int undo_state)
{
	if(IsActivated(doc)) {
		m_draw_update = true;
	}
	return FALSE;
}

void VertexMarker::OnUpdateObjectList(MQDocument doc)
{
	if(IsActivated(doc)) {
		m_draw_update = true;
	}
}

void VertexMarker::OnObjectModified(MQDocument doc)
{
	if(IsActivated(doc)) {
		m_draw_update = true;
		if(m_draw_normal) {
			m_normal_update = true;
			MQ_RefreshView(nullptr);
		}
	}
}

bool VertexMarker::empty(int boxno)
{
	if(boxno < 0 || boxno >= m_marked.size()) {
		return true;
	}
	return m_marked[boxno].empty();
}

VertexMarker::Marked::Marked() : m_markon(false), m_update(false), m_objuid(-1), m_draw(nullptr), m_color(1.0f, 0.0f, 1.0f)
{
}

VertexMarker::Marked::~Marked()
{
	m_draw = nullptr;
}

void VertexMarker::Marked::clear()
{
	m_vertices.clear();
	m_lines.clear();
	m_normal.clear();
	m_update = true;
}

bool VertexMarker::Marked::empty()
{
	return m_objuid == -1 || (m_vertices.empty() && m_lines.empty());
}

void VertexMarker::Marked::setDrawingObject(MQObject doc, MQObject draw)
{
}

// カレントオブジェクトの選択要素をboxnoにセット(または追加)する
BOOL VertexMarker::set(MQDocument doc_, int boxno, bool additional)
{
	if(boxno < 0) {
		// 指定がおかしい
		return FALSE;
	}
	if(boxno >= m_marked.size()) {
		m_marked.resize(boxno + 1);
	}
	Marked &m = m_marked[boxno];
	cmq::CMQDocument doc(doc_);
	cmq::CMQObject obj = doc.getCurrentObject();
	if(m.m_objuid == -1) {
		m.m_objuid = obj.getUniqueID();
	}
	else if(!additional || m.m_objuid != obj.getUniqueID()) {
		m.clear();
		m.m_objuid = obj.getUniqueID();
	}
	else {
	}
	// 選択頂点
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		if(vit->isSelect()) {
				m.m_vertices.push_back(obj.getVertexUniqueID(vit.index()));
		}
	}
	m.m_normal.clear();
	// 選択面/辺
	cmq::CMQContainer<cmq::CMQFace, cmq::CMQObject> fcontainer(&obj);
	for(auto fit = fcontainer.begin(); fit != fcontainer.end(); ++fit) {
		// 辺
		auto varray = obj.getFacePointArray(fit.index());
		for(int i = 0; i < varray.size(); i++) {
			if(fit->isSelect() || doc.isSelectLine(obj, fit.index(), i)) {
				cmq::vindex_t v1 = varray[i];
				cmq::vindex_t v2 = varray[NEXTIDX(i, varray.size())];
				UINT vuid1 = obj.getVertexUniqueID(v1);
				UINT vuid2 = obj.getVertexUniqueID(v2);
				cmq::lindex_t lineid = cmq::makeLineId(vuid1, vuid2); // vindex_tではなくUINTだが、このまま使う
				m.m_lines.insert(lineid);
			}
		}
	}
	m.m_markon = true;
	m.m_update = true;
	m_normal_update = true;
	return TRUE;
};

BOOL VertexMarker::clear(MQDocument doc, int boxno)
{
	if(boxno < 0) {
		// 指定がおかしい
		return FALSE;
	}
	if(boxno >= m_marked.size()) {
		m_marked.resize(boxno + 1);
	}
	Marked &m = m_marked[boxno];
	m.m_markon = false;
	m.clear();
	return TRUE;
}

void VertexMarker::setView(int boxno, bool flag)
{
	if(boxno >= 0 && boxno < m_marked.size()) {
		m_marked[boxno].m_markon = flag;
	}
	m_draw_update = true;
	m_normal_update = true;
}

void VertexMarker::selectback(MQDocument doc, int boxno, bool flag)
{
	if(boxno >= 0 && boxno < m_marked.size()) {
		Marked &m = m_marked[boxno];
		if(!m.empty()) {
			if(!flag) {
				doc->ClearSelect(MQDOC_CLEARSELECT_ALL);
			}
			cmq::CMQObject obj(doc->GetObjectFromUniqueID(m.m_objuid), doc);
			for(UINT vuid : m.m_vertices) {
				obj.addSelectVertex(obj.getVertexIndexFromUniqueID(vuid));
			}
			for(cmq::lindex_t lindex : m.m_lines) {
				std::pair<UINT, UINT> vpair = cmq::flId2FaceLinePair(lindex);
				cmq::vindex_t v1 = obj.getVertexIndexFromUniqueID(vpair.first);
				cmq::vindex_t v2 = obj.getVertexIndexFromUniqueID(vpair.second);
				auto lines = obj.getLineWithVertex(v1, v2);
				for(auto flindex : lines) {
					auto flpair = cmq::flId2FaceLinePair(flindex);
					obj.addSelectLine(flpair.first, flpair.second);
				}
			}
		}
	}
}

void VertexMarker::setView(bool flag)
{
	m_view = flag;
	m_draw_update = true;
	m_normal_update = true;
}

void VertexMarker::setDrawNormal(bool flag)
{
	m_draw_normal = flag;
	m_draw_update = true;
	m_normal_update = m_draw_update;
}

void VertexMarker::setOverwriteSelected(bool flag)
{
	m_overwrite_selected = flag;
	m_draw_update = true;
}

void VertexMarker::setViewAlways(bool flag)
{
	m_view_always = flag;
	m_draw_update = true;
}

void VertexMarker::setNormalLength(float len)
{
	m_normal_length = len;
	m_draw_update = true;
	m_normal_update = true;
}
