﻿#include "MQBasePlugin.h"

#ifndef _MainDefine
#define _MainDefine

// メインクラス
class VertexMarker : public MQStationPlugin
{
public:
	VertexMarker();
	virtual ~VertexMarker();

	virtual void GetPlugInID(DWORD *Product, DWORD *ID);
	virtual const char *GetPlugInName(void);
	virtual const char *EnumString(void);
	virtual BOOL Initialize();
	virtual void Exit();
	virtual BOOL Activate(MQDocument doc, BOOL flag);
	virtual BOOL IsActivated(MQDocument doc);

	virtual void OnNewDocument(MQDocument doc, const char *filename, NEW_DOCUMENT_PARAM& param);
	virtual void OnDraw(MQDocument doc, MQScene scene, int width, int height);
	virtual BOOL OnUndo(MQDocument doc, int undo_state);
	virtual void OnUpdateObjectList(MQDocument doc);
	virtual void OnObjectModified(MQDocument doc);

	typedef bool (VertexMarker::*ExecuteCallbackProc)(MQDocument doc);
	void Execute(ExecuteCallbackProc proc);

	BOOL set(MQDocument doc, int boxno, bool additional = false);
	BOOL clear(MQDocument doc, int boxno);

	void setView(bool flag);
	void setDrawNormal(bool flag);
	void setOverwriteSelected(bool flag);
	void setViewAlways(bool flag);
	void setNormalLength(float len);

	void setView(int boxno, bool flag);
	void selectback(MQDocument doc, int boxno, bool flag);
	bool empty(int boxno);

	// マークした頂点/辺を記録する構造体
	struct Marked
	{
		bool m_markon;
		bool m_update;
		UINT m_objuid;
		std::vector<UINT> m_vertices;
		std::unordered_set<cmq::lindex_t> m_lines;
		std::unordered_map<UINT, MQPoint> m_normal;
		MQObject m_draw;
		MQColor m_color;

		Marked();
		~Marked();
		void clear();
		bool empty();
		void setDrawingObject(MQObject doc, MQObject draw);
	};
	std::vector<Marked> m_marked;

protected:
	struct CallbackInfo
	{
		ExecuteCallbackProc proc;
	};

	// コールバックに対する実装部
	virtual bool ExecuteCallback(MQDocument doc, void *option);

private:
	VertexMarkerWindow *m_Window;
	bool m_bActivate;

	MQDocument m_doc;

	bool m_view;
	bool m_draw_update;
	bool m_draw_normal;
	bool m_normal_update;
	bool m_view_always;

	float m_normal_length;

	bool m_overwrite_selected;

	void createNormalViewObject();
	void DeleteDrawingObject_(MQDocument doc, Marked &m);

	friend class VertexMarkerWindow;
};

#endif
