﻿#include <windows.h>

#include "VertexMarkerWindow.h"
#include "VertexMarker.h"

VertexMarkerWindow::VertexMarkerWindow(VertexMarker *plugin, MQWindowBase& parent) : MQWindow(parent), m_pPlugin(plugin)
{
	setlocale(LC_CTYPE, "");
	wchar_t buffer[256] = L"VertexMarker";
	SetTitle(buffer);
	SetCanResize(false);

	MQFrame *mainFrame = CreateVerticalFrame(this);
	mainFrame->SetInSpace(0);
	mainFrame->SetOutSpace(0.2);

	for(int i = 0; i < 3; i++) {
		MQFrame *frame1 = CreateHorizontalFrame(mainFrame);
		wchar_t no[2];
		wsprintf(no, L"%d", i);
		MQButton *button_label = CreateButton(frame1, no);
		button_label->SetToggle(true);
		button_label->AddLeftDownEvent(this, &VertexMarkerWindow::viewswitch);
		button_label->AddRightDownEvent(this, &VertexMarkerWindow::selectback);

		MQButton *button_set = CreateButton(frame1, L"Set");
		button_set->AddLeftDownEvent(this, &VertexMarkerWindow::set);
		button_set->AddRightDownEvent(this, &VertexMarkerWindow::setcolor);

		MQButton *button_clear = CreateButton(frame1, L"Clr");
		button_clear->AddLeftDownEvent(this, &VertexMarkerWindow::clear);

		m_button_label.push_back(button_label);
		m_button_set.push_back(button_set);
		m_button_clear.push_back(button_clear);
	}

	{
		MQFrame *frame2 = CreateHorizontalFrame(mainFrame);
		checkView = CreateCheckBox(frame2, L"View");
		checkView->AddChangedEvent(this, &VertexMarkerWindow::viewChanged);
		checkView->SetChecked(true);

		checkNormal = CreateCheckBox(frame2, L"Normal");
		checkNormal->AddChangedEvent(this, &VertexMarkerWindow::normalChanged);
		checkNormal->SetChecked(false);
	}
	{
		MQFrame *frame3 = CreateHorizontalFrame(mainFrame);
		checkOverwrite = CreateCheckBox(frame3, L"Overwrite");
		checkOverwrite->AddChangedEvent(this, &VertexMarkerWindow::overwriteChanged);
		checkOverwrite->SetChecked(true);

		checkAlways = CreateCheckBox(frame3, L"Allways");
		checkAlways->AddChangedEvent(this, &VertexMarkerWindow::viewAlways);
		checkAlways->SetChecked(false);
	}

	spinNormalLength = CreateDoubleSpinBox(mainFrame);
	spinNormalLength->SetPosition(5.0);
	spinNormalLength->SetMin(-100.0);
	spinNormalLength->SetMax(100.0);
	spinNormalLength->SetIncrement(5.0);
	spinNormalLength->SetMaxDecimalDigit(3);
	spinNormalLength->AddChangingEvent(this, &VertexMarkerWindow::changeNormalLength);

	this->AddHideEvent(this, &VertexMarkerWindow::OnHide);
	//this->AddKeyUpEvent(this, &VertexMarkerWindow::keyevent);
}

VertexMarkerWindow::~VertexMarkerWindow()
{
}

void VertexMarkerWindow::resetall()
{
	for(int i = 0; i < 3; i++) {
		m_button_label[i]->SetDown(false);
	}
}

BOOL VertexMarkerWindow::OnHide(MQWidgetBase *sender, MQDocument doc)
{
	if(m_pPlugin->m_bActivate) {
		m_pPlugin->m_bActivate = false;
		m_pPlugin->WindowClose();
	}
	return FALSE;
}

BOOL VertexMarkerWindow::viewswitch(MQWidgetBase *parent, MQDocument doc, MQWidgetMouseParam &param)
{
	for(auto it = m_button_label.begin(); it != m_button_label.end(); ++it) {
		if(*it == parent) {
			int boxno = it - m_button_label.begin();
			MQButton *button = *it;
			bool flag = !button->GetDown();
			m_pPlugin->setView(boxno, flag);
			return TRUE;
		}
	}
	return FALSE;
}

BOOL VertexMarkerWindow::selectback(MQWidgetBase *parent, MQDocument doc, MQWidgetMouseParam &param)
{
	for(auto it = m_button_label.begin(); it != m_button_label.end(); ++it) {
		if(*it == parent) {
			int boxno = it - m_button_label.begin();
			MQButton *button = *it;
			m_pPlugin->selectback(doc, boxno, param.Ctrl);
			return TRUE;
		}
	}
	return FALSE;
}

BOOL VertexMarkerWindow::set(MQWidgetBase *parent, MQDocument doc, MQWidgetMouseParam &param)
{
	for(auto it = m_button_set.begin(); it != m_button_set.end(); ++it) {
		if(*it == parent) {
			int boxno = it - m_button_set.begin();
			if(!param.Shift && !m_pPlugin->empty(boxno)) {
				if(param.Ctrl) {
					MQDialog::DIALOG_RESULT result = MQDialog::MessageOkCancelBox(*this, L"Add?", L"Marker");
					if(result != MQDialog::DIALOG_OK) {
						return FALSE;
					}

				}
				else {
					MQDialog::DIALOG_RESULT result = MQDialog::MessageOkCancelBox(*this, L"Overwrite?", L"Marker");
					if(result != MQDialog::DIALOG_OK) {
						return FALSE;
					}
				}
			}
			BOOL result = m_pPlugin->set(doc, boxno, param.Ctrl);
			if(result) {
				m_button_label[boxno]->SetDown(true);
			}
			return result;
		}
	}
	return FALSE;
}

BOOL VertexMarkerWindow::clear(MQWidgetBase *parent, MQDocument doc, MQWidgetMouseParam &param)
{
	for(auto it = m_button_clear.begin(); it != m_button_clear.end(); ++it) {
		if(*it == parent) {
			int boxno = it - m_button_clear.begin();
			if(!param.Shift && !m_pPlugin->empty(boxno)) {
				MQDialog::DIALOG_RESULT result = MQDialog::MessageOkCancelBox(*this, L"Clear?", L"Marker");
				if(result != MQDialog::DIALOG_OK) {
					return FALSE;
				}
			}
			BOOL result = m_pPlugin->clear(doc, boxno);
			m_button_label[boxno]->SetDown(false);
			return result;
		}
	}
	return FALSE;
}

BOOL VertexMarkerWindow::setcolor(MQWidgetBase *parent, MQDocument doc, MQWidgetMouseParam& param)
{
	for(auto it = m_button_set.begin(); it != m_button_set.end(); ++it) {
		if(*it == parent) {
			int boxno = it - m_button_set.begin();
			MQColor color = m_pPlugin->m_marked[boxno].m_color;
			BOOL result = callColorPanelDialog(param.ScreenPos, color);
			if(result) {
				m_pPlugin->m_marked[boxno].m_color = color;
			}
			return result;
		}
	}
	return FALSE;
}

BOOL VertexMarkerWindow::viewChanged(MQWidgetBase *parent, MQDocument doc)
{
	bool flag = checkView->GetChecked();
	m_pPlugin->setView(flag);
	return TRUE;
}

BOOL VertexMarkerWindow::normalChanged(MQWidgetBase *parent, MQDocument doc)
{
	bool flag = checkNormal->GetChecked();
	m_pPlugin->setDrawNormal(flag);
	return TRUE;
}

BOOL VertexMarkerWindow::overwriteChanged(MQWidgetBase *parent, MQDocument doc)
{
	bool flag = checkOverwrite->GetChecked();
	m_pPlugin->setOverwriteSelected(flag);
	return TRUE;
}

BOOL VertexMarkerWindow::changeNormalLength(MQWidgetBase *parent, MQDocument doc)
{
	float len = spinNormalLength->GetPosition();
	m_pPlugin->setNormalLength(len);
	return TRUE;
}

BOOL VertexMarkerWindow::viewAlways(MQWidgetBase *parent, MQDocument doc)
{
	bool flag = checkAlways->GetChecked();
	m_pPlugin->setViewAlways(flag);
	return TRUE;
}

BOOL VertexMarkerWindow::callColorPanelDialog(const POINT &pos, MQColor &color)
{
	int r = color.r * 255;
	int g = color.g * 255;
	int b = color.b * 255;
	SetColorDialog dlg(*this, m_pPlugin, r, g, b);
	dlg.SetPosX(pos.x);
	dlg.SetPosY(pos.y);
	if(dlg.Execute() == MQDialog::DIALOG_OK) {
		dlg.m_color->GetColor(r, g, b);
		color.r = (float)r / 255.0;
		color.g = (float)g / 255.0;
		color.b = (float)b / 255.0;
		return TRUE;
	}
	return FALSE;
}

SetColorDialog::SetColorDialog(MQWindowBase &parent, VertexMarker *plugin, int r, int g, int b) : MQDialog(parent), m_pPlugin(plugin)
{
	MQFrame *frame1 = CreateVerticalFrame(this);

	m_color = CreateColorPanel(frame1);
	m_color->SetColor(r, g, b);

	MQFrame *frame2 = CreateHorizontalFrame(frame1);
	MQButton *button_ok = CreateButton(frame2, L"OK");
	button_ok->SetModalResult(DIALOG_OK);
	button_ok->SetDefault(true);
	MQButton *button_cancel = CreateButton(frame2, L"Cancel");
	button_cancel->SetModalResult(DIALOG_CANCEL);
	button_cancel->SetCancel(true);
}
